package org.algorithm.core.remoteModel;

import org.algorithm.core.cnn.AlgorithmCNNExecutor;
import org.algorithm.core.cnn.entity.Lemma;
import org.algorithm.core.cnn.entity.Triad;
import org.algorithm.util.HttpGetAndPost;
import org.diagbot.pub.utils.PropertiesUtil;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: bijl
 * @Date: 2019/5/21 11:02
 * @Decription: 关系抽取模型接口，从网络中获取
 */
public class RelationExtractionModelFromHttp extends AlgorithmCNNExecutor {
    @Override
    public List<Triad> execute(String content, List<Lemma> lemmas) {
        String url = getUrl();
        String positions = makePositionsParam(lemmas);
        String sentence = content;
        String indexPairAndRelations = HttpGetAndPost.sendPost(url,
                "sentence="+sentence+"&positions="+positions);
        return mergePredictInfoToTriads(indexPairAndRelations, lemmas);  // 合并信息到三元组中去
    }

    /**
     * 用lemma内部属性position组成字符串，按lemma在lemmas中的顺序
     * @param lemmas 实体list
     * @return
     */
    private String makePositionsParam(List<Lemma> lemmas){
        String results = "";
        for(Lemma lm: lemmas)
            results += lm.getPosition() + "|";  // 形式：1,2|33,45|
        results = results.substring(0, results.length() - 1);  // 形式：1,2|33,45
        return results;
    }

    /**
     * 获取url
     * @return
     */
    private String getUrl(){
        PropertiesUtil prop = new PropertiesUtil("/algorithm.properties");
        return prop.getProperty("relationExtractionUrl");
    }

    /**
     * 合并预测信息到三元组中去
     * @param indexPairAndRelations 预测信息，来自网络 // 形式：1,2:有|3,5:无
     * @param lemmas 实体
     * @return 有关系的实体对
     */
    private List<Triad> mergePredictInfoToTriads(String indexPairAndRelations, List<Lemma> lemmas){
        List<Triad> triads = new ArrayList<>();
        String[] posRelationArray = indexPairAndRelations.split("\\|");
        String[] info;
        String[] indexPair;
        int index1;
        int index2;
        for(String posRelation:posRelationArray){
            info = posRelation.split(":");  // 形式：1,2:有
            if ("有".equals(info[1])){  // 仅返回有关系的
                indexPair = info[0].split(","); // 形式：1,2
                index1 = Integer.parseInt(indexPair[0]);
                index2 = Integer.parseInt(indexPair[1]);
                Triad triad = new Triad();
                triad.setL_1(lemmas.get(index1));
                triad.setL_2(lemmas.get(index2));
                triad.setRelation(info[1]);
                triads.add(triad);
            }
        }

        return triads;
    }
}

